"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var removeList = [
    'parent',
    '_children',
    'statements',
    'pos',
    'end',
    'modifierFlagsCache',
    'transformFlags',
    'flowNode',
    'parent',
];
function syntaxKindToName(kind) {
    return ts.SyntaxKind[kind];
}
exports.syntaxKindToName = syntaxKindToName;
function flagsToText(kind) {
    return ts.TypeFlags[kind];
}
exports.flagsToText = flagsToText;
/** True if this is visible outside this file, false otherwise */
function isNodeExported(node) {
    // Parse the modifier array for the export keyword. If it is found
    // and the node.parent is a sourcefile, return true
    // This only returns top level exports
    var modifiers = node.modifiers;
    if (modifiers) {
        for (var i = 0; i < modifiers.length; i++) {
            if (modifiers[i].kind === ts.SyntaxKind.ExportKeyword) {
                return node.parent.kind === ts.SyntaxKind.SourceFile;
            }
        }
    }
    return false;
}
function symbolMapToString(map) {
    var values = [];
    map.forEach(function (value, key) {
        values.push(value.name);
    });
    return values.join('; ');
}
function simplePrint(checker, node, indent) {
    if (indent === void 0) { indent = 0; }
    var indentText = Array(indent * 4).join(' ');
    var info = [];
    function addSymbol(info, node, prefix) {
        if (prefix === void 0) { prefix = ''; }
        var s = checker.getSymbolAtLocation(node);
        if (!s) {
            return;
        }
        if (s.exports && s.exports.size > 0) {
            info.push(prefix + 'exports: ' + symbolMapToString(s.exports));
        }
        if (s.globalExports && s.globalExports.size > 0) {
            info.push(prefix + 'globalExports: ' + symbolMapToString(s.exports));
        }
        info.push(prefix + 'name: ' + s.name);
        var type = checker.getTypeOfSymbolAtLocation(s, node);
        if (type && type.symbol) {
            info.push(prefix + 'type: ' + type.symbol.name);
        }
        var comments = s.getDocumentationComment();
        if (comments.length > 0) {
            info.push(prefix + 'comment: \'' + comments.map(function (i) { return i.text; }).join('; ') + '\'');
        }
    }
    if (node.kind === ts.SyntaxKind.FunctionDeclaration) {
        var d = node;
        info.push('name: ' + d.name.text);
        info.push('type: ' + d.type.getText());
    }
    if (node.kind === ts.SyntaxKind.ClassDeclaration) {
        var d = node;
        info.push('name: ' + d.name.text);
        info.push('members: [' + d.members.map(function (i) { return i.name ? i.name.getText() : "NAME_UNDEFINED"; }).join(', ') + ']');
    }
    if (node.kind === ts.SyntaxKind.Identifier) {
        var d = node;
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.InterfaceDeclaration) {
        var d = node;
        addSymbol(info, node);
        addSymbol(info, d.name, 'name-');
    }
    if (node.kind === ts.SyntaxKind.ExportDeclaration) {
        var d = node;
        info.push('name: ' + d.name);
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.ExportAssignment) {
        var d = node;
        info.push('name: ' + d.name);
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.VariableDeclaration) {
        var d = node;
        addSymbol(info, d.name, 'name-');
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.VariableStatement) {
        var d = node;
        if (d.declarationList && d.declarationList.declarations.length === 1) {
            var dec = d.declarationList.declarations[0];
            info.push('declarations: ' + ("(kind: " + syntaxKindToName(dec.kind) + ", name: " + dec.name.getText() + ", name.kind: " + syntaxKindToName(dec.name.kind) + ")"));
            if (dec.type) {
                info.push('declaration.type: ' + syntaxKindToName(dec.type.kind));
            }
            if (dec.initializer) {
                info.push('declaration.initializer: ' + syntaxKindToName(dec.initializer.kind));
            }
        }
        addSymbol(info, node);
    }
    if (node.kind === ts.SyntaxKind.CallExpression) {
        var d = node;
        info.push('arguments: ' + d.arguments.map(function (i) { return i.getText(); }).join(';'));
    }
    if (isNodeExported(node)) {
        info.push('exported: true');
    }
    var infoText = info.length === 0 ? '' : "(" + info.join(', ') + ")";
    console.log("" + indentText + syntaxKindToName(node.kind) + " " + infoText);
    ts.forEachChild(node, function (child) {
        simplePrint(checker, child, indent + 1);
    });
}
exports.simplePrint = simplePrint;
//# sourceMappingURL=printUtils.js.map